<?php

declare(strict_types=1);

namespace FiloBlu\Checkproducts\Console\Command;

use FiloBlu\Checkproducts\Helper\Email;
use FiloBlu\Checkproducts\Model\CollectorInterfaceFactory;
use FiloBlu\Checkproducts\Model\ProductCheckerInterfaceFactory;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 *
 */
class CheckProducts extends Command
{
    /**
     * @var \FiloBlu\Checkproducts\Model\ProductCheckerInterfaceFactory
     */
    private $productCheckerFactory;
    /**
     * @var \FiloBlu\Checkproducts\Console\Command\CollectorInterfaceFactory
     */
    private $collectorFactory;
    /**
     * @var \FiloBlu\Checkproducts\Helper\Email
     */
    private $emailHelper;

    /**
     * @param \FiloBlu\Checkproducts\Model\ProductCheckerInterfaceFactory $productCheckerFactory
     * @param \FiloBlu\Checkproducts\Model\CollectorInterfaceFactory $collectorFactory
     * @param \FiloBlu\Checkproducts\Helper\Email $emailHelper
     * @param string|null $name
     */
    public function __construct(
        ProductCheckerInterfaceFactory $productCheckerFactory,
        CollectorInterfaceFactory $collectorFactory,
        Email $emailHelper,
        string $name = null
    ) {
        parent::__construct($name);
        $this->productCheckerFactory = $productCheckerFactory;
        $this->collectorFactory = $collectorFactory;
        $this->emailHelper = $emailHelper;
    }

    protected function configure()
    {
        $this->setName('filoblu:check:products')
            ->setDescription('Check Products');
    }

    /**
     * @param \Symfony\Component\Console\Input\InputInterface $input
     * @param \Symfony\Component\Console\Output\OutputInterface $output
     * @return int
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\MailException
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $collector = $this->collectorFactory->create();
        $this->productCheckerFactory->create()->check($collector);
        $this->emailHelper->sendReport($collector);
        return 0;
    }
}